:-module(draw, [draw/1]).

/*-----------------------------------------------------------------------------*/
/*  This program was written by Mark Holcomb and is hearby released into the   */
/*  public domain provided this header remains intact.  As usual, no guarantee */
/*  is made as to the correctness of the code, the suitability of this code    */
/*  for any particular application, or that the code will be maintained.       */
/*  USAGE:                                                                     */
/*      The code expects (as input), the tree structure of a sentence in a     */
/*      form similar to:   s(np(d(the),n(dog)),vp(v(ran),np(d(the),n(house)))) */
/*      and prints an ascii diagram of the structure.                          */
/*                                                                             */
/*        eg.  draw(s(np(d(the),n(dog)),vp(v(ran),np(d(the),n(house))))).      */
/*									       */
/*                       s                                                     */
/*		         |                                                     */
/*          +-----------------+                                                */
/*    	    np                vp                                               */
/*	    |                 |                                                */
/*	 +------+      +----------+                                            */
/*	 d      n      v          np                                           */
/*	 |      |      |          |                                            */
/*	 |      |      |      +-------+                                        */
/*	 |      |      |      d       n                                        */
/*	 |      |      |      |       |                                        */
/*	 |      |      |      |       |                                        */
/*      the    dog    ran    the    house                                      */
/*                                                                             */
/*-----------------------------------------------------------------------------*/
/* ---- begin draw routine ----------------------------------------------------*/

/*29*/ :- ensure_loaded(library(lists)).

/*31*/ /* the 5 in add is the constant offset from the left edge of the screen */
/*32*/ 
/*33*/ draw(Struct) :- 
/*34*/ 	md(1,Struct,1,Depth),		
/*35*/ 	add(Depth,Struct,New_struct,5,_),		
/*36*/ 	Mod is Depth + 1,		
/*37*/ 	breadth(1,Mod,New_struct),
        !.    %% MVolk

/*39*/ trunc(X,Y) :-	
/*40*/ 	name(X,NX),	
/*41*/ 	until_dot(NX,NY),	
/*42*/ 	reverse(NY,RNY),	
/*43*/ 	name(Y,RNY).
 
/*45*/ until_dot(X,Res) :- 
/*46*/ 	until_dot_aux(X,[],Res).

/*48*/ until_dot_aux([],Res,Res) :- !.

/*50*/ until_dot_aux([46|_],Res,Res) :- !.

/*52*/ until_dot_aux([H|T],Sofar,Res) :- 
/*53*/ 	until_dot_aux(T,[H|Sofar],Res).

/* append([],X,X).

   append([X1|X2],Y,[X1|Z]) :- 
     append(X2,Y,Z).
*/

/*59*/ wspaces(0).
/*60*/ wspaces(Num) :- 
/*61*/ 	New is Num - 1, write(' '), 
/*62*/ 	wspaces(New).
/*63*/ 
/*64*/ spaces(0,Tout,Tout).
/*65*/ spaces(Num,Tin,Tout) :- 
/*66*/ 	New is Num - 1,	
/*67*/ 	append(Tin,[32],Tmid),
/*68*/ 	spaces(New,Tmid,Tout).
/*69*/ 
/*70*/ aux_lines(_,_,Max,Max) :- !. 

/*72*/ /* if this is the last level, don't print |'s */
/*73*/ 
/*74*/ aux_lines(Out,Begin,_,_) :- 
/*75*/ 	lines(Out,Begin).

/*77*/ lines([],_).
/*78*/ 
/*79*/ lines([H|T],Cur) :- 
/*80*/ 	New is H - Cur, 
/*81*/ 	wspaces(New),
/*82*/  	Next is H + 1,
/*83*/ 	write('|'), 
/*84*/ 	lines(T,Next). 

/*86*/ size(Term,Size) :- 
/*87*/ 	name(Term,List),
/*88*/ 	len(List,Size). 
/*89*/ 
/*90*/ len([],0).
/*91*/ 
/*92*/ len([_|T],Length) :- 
/*93*/ 	len(T,J), 
/*94*/ 	Length is J + 1.
/*95*/ 
/*96*/ dummy(~,Tin,Tout) :- !,
/*97*/        append(Tin,[124],Tout). 
/*98*/ 
/*99*/ dummy(Name,Tin,Tout) :- 
/*100*/ 	name(Name,List), 
/*101*/ 	append(Tin,List,Tout).
/*102*/ 
/*103*/ prt([]).
/*104*/ prt(List) :- 
/*105*/ 	name(Text,List),
/*106*/ 	write(Text).
/*107*/ 
/*108*/ post(0,_) :- !.
/*109*/ post(1,_) :- !. 
/*110*/ 
/*111*/ post(_,Dout) :- 
/*112*/ 	prt(Dout).
/*113*/ 
/*114*/ end([],[],[]):- !. /* empty list sent to end */
/*115*/ 
/*116*/ end([H],[],H) :- !,atomic(H). /* one element list */
/*117*/ 
/*118*/ end([H|T],[H|Out],End) :- 
/*119*/ 	end(T,Out,End).
/*120*/ 
/*121*/ cross(2,Din,Dout,Chars) :- !, 
/*122*/ 	single(Din,Dout,Chars).
/*123*/ 
/*124*/ cross(_,Din,Dout,Chars) :- 
/*125*/ 	aux_cross(Din,Dout,Chars).
/*126*/ 
/*127*/ ifs([],Dout,Dout) :- !.
/*128*/ ifs(32,Dout,Dout) :- !,
/*129*/ 	nl,
/*130*/ 	write('Error: blank end in ifs ').
/*131*/ 
/*132*/ ifs(End,Din,Dout) :- 
/*133*/ 	append(Din,[End],Dout).
/*134*/ 
/*135*/ single(Din,Dout,Chars) :- 
/*136*/ 	end(Din,Dmid,End),  
%/*137*/   if [] add char - 1 spaces 
%/*138*/ if + append + and char - 1 spaces if | append | and char - 1 space 
%/*139*/ if anything else, I messed up 
/*140*/ 	ifs(End,Dmid,Dmid2),
/*141*/ 	New is Chars - 1,
/*142*/ 	spaces(New,Dmid2,Dmid3),
/*143*/ 	append(Dmid3,[124],Dout). 
/*144*/ 
/*145*/ /* add a '|' to Dout */
/*146*/ 
/*147*/ dashes(0,Dout,Dout) :- !.
/*148*/ 
/*149*/ dashes(Num,Din,Dout) :- New is Num - 1,
/*150*/ 			append(Din,[45],Dmid),
/*151*/ 			dashes(New,Dmid,Dout). 
/*152*/ 
/*153*/ cont(32,Chars,Din,Dout) :- !,dashes(Chars,Din,Dout).		
/*154*/ 	/* if ' ' append char dashes */
/*155*/ 
/*156*/ cont(End,Chars,Din,Dout) :- 	
/*157*/ 	New is Chars - 1,
/*158*/ 	ifs(End,Din,Dmid),
/*159*/ 	spaces(New,Dmid,Dmid1),
/*160*/ 	append(Dmid1,[43],Dout).
/*161*/ 
%/*162*/ /* if [] add char - 1 spaces and +if + append + and char - 1 spaces and +
%/*163*/ if | append | and char - 1 spaces and +	if anything else, I messed up */
/*164*/ 
/*165*/ aux_cross(Din,Dout,Chars) :- 	
/*166*/ 	end(Din,Dmid1,End),	
/*167*/ 	cont(End,Chars,Dmid1,Dmid2),
/*168*/ 
/*169*/ /* since we are in a --- always end with a ' ' */
/*170*/ 	append(Dmid2,[32],Dout).
/*171*/ 
/*172*/ choose(32,Din,Dout) :- append(Din,[43],Dout).
/*173*/ choose(End,Din,Dout) :-  ifs(End,Din,Dout).
/*174*/ 
/*175*/ check(In) :- In>0,!.
/*176*/ 
/*177*/ check(In) :- In=<0,
/*178*/ write('Error: label overlap detected; suggest increasing base node seperation in loc'),
/*179*/ 	nl, fail.
/*180*/ 
/*181*/ breadth(Max,Max,_) :- !.
/*182*/ breadth(Level,Max,Struct) :-
/*183*/ 	New is Level + 1,
/*184*/ 	at(Level,1,Struct,0,_,[],Out,[],Tout,[],Dout),
/*185*/ 	/* print dashes, nl, text, nl, lines, nl */
/*186*/ 	post(Level,Dout), nl,
/*187*/ 	prt(Tout), nl,	
/*188*/ 	aux_lines(Out,0,New,Max),nl,
/*189*/ 	breadth(New,Max,Struct).
/*190*/ 
/*191*/ at(1,Arity,Struct,Cur,Pos,In,Out,Tin,Tout,Din,Dout) :- !,
/*192*/ 	functor(Struct,Name,_),	
/*193*/ 	arg(1,Struct,Spaces),
/*194*/ 	Actual is Spaces - Cur,	
/*195*/ 	check(Actual),
/*196*/ 	spaces(Actual,Tin,Tmid),
/*197*/ 	T is Cur + Actual,	
/*198*/ 	size(Name,Size), 
/*199*/ 	Pos is T + Size,
/*200*/ 	M is (Size+1)/2, 
/*201*/ 	trunc(M,V),
/*202*/ 	L is T + V - 1,
/*203*/ 	len(Din,Len),
/*204*/ 	Chars_needed is ((L + 1) - Len),
/*205*/ 	cross(Arity,Din,Dout,Chars_needed),
/*206*/ 	append(In,[L],Out),
/*207*/ 	dummy(Name,Tmid,Tout).
/*208*/ 
/*209*/ at(Level,_,Struct,Cur,Pos,In,Out,Tin,Tout,Din,Dout) :- 
/*210*/ 	functor(Struct,_,Arity),
/*211*/ 	Next_lev is Level - 1,	
/*212*/ 	for_each(1,Arity,Struct,Next_lev,Cur,Pos,In,Out,Tin,Tout,Din,Dout).
/*213*/ 
/*214*/ for_each(Same,Same,_,_,X,X,Y,Y,Z,Z,Din,Dout) :- !, end(Din,Mid,End),	
/*215*/ 	choose(End,Mid,Dout).	
/*216*/ /* if End is ' ' then append '+'else append End */
/*217*/ 
/*218*/ for_each(Begin,End,Struct,Level,Cur,New_Pos,In,Out,Tin,Tout,Din,Dout) :-
/*219*/ 	Count is Begin + 1,
/*220*/ 	arg(Count,Struct,Sub),
/*221*/ 	at(Level,End,Sub,Cur,Pos,In,Mid,Tin,TMid,Din,Dmid),
/*222*/ 	for_each(Count,End,Struct,Level,Pos,New_Pos,Mid,Out,TMid,Tout,Dmid,Dout).
/*223*/ 
/*224*/ /* ------ end draw  ------------- max depth begin --------------------------*/
/*225*/ 
/*226*/ md(Level,Struct,Cur,Level) :- 
/*227*/ 	functor(Struct,_,0), 
/*228*/ 	Level >= Cur, !.
/*229*/ 
/*230*/ md(Level,Struct,Cur,Cur) :- 
/*231*/ 	functor(Struct,_,0), 
/*232*/ 	Cur < Level, !.
/*233*/ 
/*234*/ md(Level,Struct,Cur,Max) :- 
/*235*/ 	functor(Struct,_,Arity),
/*236*/ 	Next is Level + 1,
/*237*/ 	foreach(0,Arity,Struct,Cur,Max,Next).
/*238*/ 
/*239*/ foreach(Same,Same,_,Max,Max,_) :- !.
/*240*/ 
/*241*/ foreach(Begin,End,Struct,Cur,Max,Level) :- 
/*242*/ 	Count is Begin + 1,
/*243*/ 	arg(Count,Struct,Sub),
/*244*/ 	md(Level,Sub,Cur,Cmax),
/*245*/         foreach(Count,End,Struct,Cmax,Max,Level).
/*246*/ 
/*247*/ /*--- max depth end ----------------------------------------------------------*/
/*248*/ /* the 4 in "New is (Cur + Size) + 4" below is the spacing between base nodes */  
/*249*/ /*of the tree. If you are having label overlap problems, increase it and      */
/*250*/ /*   they might go away */
/*251*/ 
/*252*/ loc(Struct,Cur,New,New_struct,Adj) :-   
/*253*/ 	size(Struct,Size),
/*254*/ 	New is (Cur + Size) + 4,
/*255*/ 	functor(Struct,Name,Arity),
/*256*/ 	NArity is Arity + 1,
/*257*/ 	functor(New_struct,Name,NArity),
/*258*/ 	M is (Size + 1)/2, trunc(M,V),
/*259*/ 	Adj is Cur + (V - 1). 
/*260*/ 
/*261*/ al(0,Struct,New_struct,Cur,New,Adj) :- 
/*262*/ 	loc(Struct,Cur,New,New_struct,Adj),
/*263*/ 	arg(1,New_struct,Cur).
/*264*/ 
/*265*/ /* everytime I get to bottom of branch, and level is not equal to maxdepth,*/
/*266*/  /* add a level */
/*267*/ 	
/*268*/ al(Needed_Depth,Struct,New_struct,Cur,Npos,Adj)  :- 
/*269*/ 	New is Needed_Depth - 1, 
/*270*/ 	al(New,Struct,Tstruct,Cur,Npos,Adj),
/*271*/ 	functor(New_struct,'~',2),
/*272*/ 	arg(2,New_struct,Tstruct),		
/*273*/ 	arg(1,New_struct,Adj).
/*274*/ 
/*275*/ /* this is the case where we are at the leaf and it is the right depth */
/*276*/ 	
/*277*/ add(1,Struct,New_struct,Cur,New) :- !,
/*278*/ 	loc(Struct,Cur,New,New_struct,_),
/*279*/ 	arg(1,New_struct,Cur).
/*280*/ 
/*281*/ /*this is the case where the leaf is not deep enough (add dummys) */
/*282*/ 
/*283*/ add(Depth,Struct,New_struct,Cur,New)  :- 
/*284*/ 	functor(Struct,_,0),!,Next is Depth - 1,			
/*285*/ 
/*286*/ /* insert add dummy structs here and return the result as New_struct */
/*287*/ 	al(Next,Struct,New_struct,Cur,New,_).
/*288*/ 
/*289*/ add(Depth,Struct,New_struct,Cur,New) :- 
/*290*/ 	functor(Struct,_,Arity),
/*291*/ 	Next is Depth - 1,
/*292*/ 	fore(0,Arity,Struct,Next,New_struct,Cur,New,_,_).
/*293*/ 
/*294*/ fore(Same,Same,Struct,_,New_struct,Cur,Cur,First,Size) :- !,
/*295*/ 				functor(Struct,Name,Arity), 						
/*296*/ 				NArity is Arity + 1,
/*297*/ 				functor(New_struct,Name,NArity),
 
/*the pos of this struct is (first + cur)/2 or first depending on single  or multiple sub structures */
/*300*/ 			size(Name,Nsize),
/*301*/ 			case(Same,Cur,Size,First,Pos,Nsize),
/*302*/ 			 arg(1,New_struct,Pos).
/*303*/ 
/*304*/ fore(Begin,End,Struct,Cur_depth,New_struct,Cur,New,Ifst,Isize) :-
/*305*/  						Count is Begin + 1,
/*306*/ 						arg(Count,Struct,Sub),
/*307*/ 						add(Cur_depth,Sub,Sub1,Cur,Mid),							arg(1,Sub1,Spos),										functor(Sub1,Name,_),									size(Name,Nsize),									first(Count,Spos,Isize,Nsize,Size,Ifst,First),	fore(Count,End,Struct,Cur_depth,New_struct,Mid,New,First,Size),						Newcount is Count + 1,						arg(Newcount,New_struct,Sub1).

first(1,Cur,_,Size,Size,_,Cur) :-!.
first(_,_,Size,_,Size,Same,Same).

/* single or multiple subargs; if End is 1, then single, else multiple */ 

case(1,_,Size,First,Pos,Nsize) :- !,
	T is First + ((Size+1)/2),				
	trunc(T,V), 
	Mid is V - 1, 				
	T2 is ((Nsize+1)/2) - 1,				
	trunc(T2,T3),				
	Pos is Mid - T3.

/*310*/ case(_,Cur,_,First,Pos,_) :- 
/*311*/ 	Temp is ((Cur - 4) + First + 1)/2,
/*312*/ 	trunc(Temp,V), Pos is V - 1.


/* ***
reverse(Input, RevInput) :-
	aux_rev(Input, [], RevInput).

aux_rev([F|R], Stack, Result) :-
	aux_rev(R, [F|Stack], Result).

aux_rev([], Result, Result).
*** */


