:-module(features, [feature/2, add_featurelist_to_matrix/2,
                   create_matrix/2, update_feature/2,
                   get_feature/2, last_matrixnr/1]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% feature.pl - erstellt und bearbeitet Feature-Matricen:
% - feature(MatrixNr, Feature=Value)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:-use_module(linkinfo, [is_headfeature/1]).

:-dynamic(last_matrixnr/1).
:-dynamic(feature/2).


%--------------------------------+
% make_new_matrixnr(-NewMatrixNr)
%--------------------------------+

make_new_matrixnr(NewNr):-   % wenn schon eine LastNr vorhanden
	last_matrixnr(LastNr),
	NewNr is LastNr +1,
	retract(last_matrixnr(LastNr)),
	assertz(last_matrixnr(NewNr)),
	!.

make_new_matrixnr(0):-   % MatrixNr initialisieren
	assertz(last_matrixnr(0)).

%---------------------------------------+
% create_matrix(+FeatureList, -MatrixNr)
%---------------------------------------+

create_matrix(FeatureList, MatrixNr):-
	make_new_matrixnr(MatrixNr),
	add_featurelist_to_matrix(MatrixNr, FeatureList).

%---------------------------------------------------+
% add_featurelist_to_matrix(+MatrixNr, +FeatureList)
%---------------------------------------------------+

add_featurelist_to_matrix(_MatrixNr, []).
add_featurelist_to_matrix(MatrixNr,
                          [FirstFeature=FirstValue|RestFeatureList]):-
	feature(MatrixNr, FirstFeature=_),   % Feature vorhanden
	update_feature(MatrixNr, FirstFeature=FirstValue),
	add_featurelist_to_matrix(MatrixNr, RestFeatureList).

add_featurelist_to_matrix(MatrixNr,
                          [FirstFeature|RestFeatureList]):-
	assertz(feature(MatrixNr, FirstFeature)),   % neues Feature
	add_featurelist_to_matrix(MatrixNr, RestFeatureList).

%---------------------------------------------+
% update_feature(+MatrixNr, +Feature=NewValue)
%---------------------------------------------+

update_feature(MatrixNr, Feature=NewValue):-   % feature vorhanden
	retract(feature(MatrixNr, Feature=_OldValue)),
	assertz(feature(MatrixNr, Feature=NewValue)).

update_feature(MatrixNr, Feature=Value):-   % zur Sicherheit: neues
                                            % Feature
	assertz(feature(MatrixNr, Feature=Value)).

%-------------------------------------------+
% get_feature(+FeatureList, ?Feature=?Value)
%-------------------------------------------+
% Fail if feature or value resp. not available

get_feature([Feature=Value|_], Feature=Value).

get_feature([_|Rest], Feature=Value):-
	get_feature(Rest, Feature=Value).