:-module(linkfeatures_basic, [rootlink/2, typefeatures/4, subsfeatures/4,
                        projectable/1, footfeature/1, relink/0,
                        tagfeatures/2, treelevel/5,
                        toplevel/1, headfeature/1]). 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% linkfeatures_basic.pl
% the rule set module: here, all the information how links shall be
% converted into constituents is stored.
% adaptions need only be made in this module.
% NB. this file is the most basic version of a rule set for Link2Tree
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% April 2002
% Stefan Hoefler
% shoefler@cl.unizh.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:-use_module(linkage2links, [link/4, token/2, add_link/4,
                             remove_link/4, add_token/3,
                             remove_token/2, add_features/2,
                             rightendpos/3, leftendpos/3,
                             remote_head/4, sub_root/2, new_nr/2]).
:-use_module(features, [feature/2]).

%---------------------------------------------------------------------+
% treelevel(TypeOfProjection, Recursive(nil/stack/flat), obligatory/optional,
% Next(Deeper)TypeOfProjection, AdditionalFeaturesListOfTheProjection)
%---------------------------------------------------------------------+

treelevel(xbar, stack, optional, [], [bar=1]).

%------------------------------+
% toplevel(TopTypeOfProjection)
%------------------------------+

toplevel(xbar).

%-----------------------------------------------+
% rootlink(RootlinkType, RootlinkSubscriptsList)
%-----------------------------------------------+

rootlink('SFI', _AnySubscripts).
rootlink('SI', _AnySubscripts).
rootlink('SF', _AnySubscripts).
rootlink('S', _AnySubscripts).

%----------------------+
% projectable(Category)
%----------------------+

projectable(_).   % no minors

%------------------------------------------------+
% typefeatures(LinktypeCapitals, HeadFeatureList,
% DependentFeatureList, ProjectionType)
%------------------------------------------------+

typefeatures(_, [], [], xbar). % no additional information specified

%------------------------------------------------------+
% subsfeatures(+Subscript, +TypeList, -HeadFeatureList,
% -DependendtFeatureList)
%------------------------------------------------------+

subsfeatures(_, [_], [], []). % default

%---------------------+
% headfeature(Feature)
%---------------------+

headfeature(bar).
headfeature(num).
headfeature(rel).

% program inherent features
% do not remove!

headfeature(cat).
headfeature(hd).
headfeature(position). % do not remove this line!
headfeature(trace).    % do not remove this line!


%---------------------+
% footfeature(Feature)
%---------------------+

footfeature(wh).

%------------------------------------------------+
% tagfeatures(TagCharactersList, TagFeautureList)
%------------------------------------------------+
%
% attention: for affixes that begin with the same letter: write clause
% for the longer affix first
%
% source: extrans_1.7/link-4.1/data21/README_NEWTAGS_WORDS2.1

tagfeatures(['a'|_], [cat=a, bar=0]).
tagfeatures(['c'|_], [cat=c, bar=0]).
tagfeatures(['d'|_], [cat=d, bar=0]).
tagfeatures(['e'|_], [cat=adv, bar=0]).
tagfeatures(['n'|_], [cat=n, bar=0]).
tagfeatures(['o'|_], [cat=p, bar=0]).
tagfeatures(['p'|_], [cat=d, bar=0]).
tagfeatures(['v'|_], [cat=v, bar=0]).
tagfeatures(_,[]). % default: do not delete!

%----------------------------+
% relink
%----------------------------+
%
% ATTENTION: order of relink-rules may be significant

relink. % default. do not remove this line.