:-module(linkfeatures_cov_corr, [rootlink/2, typefeatures/4, subsfeatures/4,
						projectable/1, footfeature/1, relink/0,
						tagfeatures/2, treelevel/5,
						toplevel/1, headfeature/1]). 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% linkfeatures.pl
% the "grammar" module: here, all the information how links shall be
% converted into constituents is stored.
% adaptions need only be made in this module.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Mai 2001
% Stefan Hoefler
% shoefler@cl.unizh.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:-use_module(linkage2links, [link/4, token/2, add_link/4,
							 remove_link/4, add_token/3,
							 remove_token/2, add_features/2,
							 rightendpos/3, leftendpos/3,
							 remote_head/4, sub_root/2, new_nr/2]).
:-use_module(features, [feature/2]).

%---------------------------------------------------------------------+
% treelevel(TypeOfProjection, Recursive(nil/stack/flat), obligatory/optional,
% Next(Deeper)TypeOfProjection, AdditionalFeaturesListOfTheProjection)
%---------------------------------------------------------------------+

treelevel(xbar, stack, optional, [], [bar=1]).

%------------------------------+
% toplevel(TopTypeOfProjection)
%------------------------------+

toplevel(xbar).

%-----------------------------------------------+
% rootlink(RootlinkType, RootlinkSubscriptsList)
%-----------------------------------------------+

rootlink('SFI', _AnySubscripts).
rootlink('SI', _AnySubscripts).
rootlink('SF', _AnySubscripts).
rootlink('S', _AnySubscripts).
%rootlink('W', _AnySubscripts).

%----------------------+
% projectable(Category)
%----------------------+

projectable(_).   % no minors
projectable(a).   % adjective phrases
projectable(adv). % adverbial phrases
projectable(c).   % complementizer phrases
projectable(d).   % determiner phrases
projectable(i).   % inflectional phrases
projectable(n).   % noun phrases
projectable(p).   % prepositional phrases
projectable(v).   % verb phrases
%projectable(w).   % wall phrases

%------------------------------------------------+
% typefeatures(LinktypeCapitals, HeadFeatureList,
% DependentFeatureList, ProjectionType)
%------------------------------------------------+

% linktypes defined by Link Grammar 4.1
typefeatures('Trace', [], [], trace).
typefeatures(_, [], [], xbar).




%------------------------------------------------------+
% subsfeatures(+Subscript, +TypeList, -HeadFeatureList,
% -DependendtFeatureList)
%------------------------------------------------------+

subsfeatures('p', ['O'], [], [num=pl]).
subsfeatures('p', [_AnyOtherType], [num=pl], [num=pl]).
subsfeatures('s', ['O'], [], [num=sg]).
subsfeatures('s', [_AnyOtherType], [num=sg], [num=sg]).

%---------------------+
% headfeature(Feature)
%---------------------+

headfeature(bar).
headfeature(cat).
headfeature(hd).
headfeature(num).
headfeature(rel).
headfeature(position). % do not remove this line!
headfeature(trace).    % do not remove this line!


%---------------------+
% footfeature(Feature)
%---------------------+

footfeature(wh).

%------------------------------------------------+
% tagfeatures(TagCharactersList, TagFeautureList)
%------------------------------------------------+
%
% attention: for affixes that begin with the same letter: write clause
% for the longer affix first
%
% source: extrans_1.7/link-4.1/data21/README_NEWTAGS_WORDS2.1

tagfeatures(['a'|_], [cat=a, bar=0]).
tagfeatures(['c'|_], [cat=c, bar=0]).
tagfeatures(['d'|_], [cat=d, bar=0]).
tagfeatures(['e'|_], [cat=adv, bar=0]).
tagfeatures(['n'|_], [cat=n, bar=0]).
tagfeatures(['o'|_], [cat=p, bar=0]).
tagfeatures(['p'|_], [cat=d, bar=0]).
tagfeatures(['v'|_], [cat=v, bar=0]).
tagfeatures(_,[]). % default: do not delete!

%----------------------------+
% relink
%----------------------------+
%
% ATTENTION: order of relink-rules may be significant

% links that violate projectivity

% AA - change of direction
relink:-
	link(HeadNr, DepNr, 'AA', Subs),
	remove_link(HeadNr, DepNr, 'AA', Subs),
	add_link(DepNr, HeadNr, 'AA', Subs).

% AF - ok

% AL
% - conflicts with the single head constraint
% - remove
relink:-
	link(HeadNr, DepNr, 'AL', Subs),
	remove_link(HeadNr, DepNr, 'AL', Subs).

% AM
% - conflicts with the root word constraint
% - change of direction
relink:-
	link(HeadNr, DepNr, 'AM', Subs),
	remove_link(HeadNr, DepNr, 'AM', Subs),
	add_link(DepNr, HeadNr, 'AM', Subs).

% AN - ok
% AZ - ok

% B - 

% BIq - relink
relink:-
	link(BIHead, WH, 'BI', ['q'|RestSubs]),
	remove_link(BIHead, WH, 'BI', ['q'|RestSubs]),
	sub_root(WH, SubRoot),
	add_link(BIHead, SubRoot, 'BI', ['q'|RestSubs]).

% BT - relink
relink:-
	link(Last, Years, 'BT', BTSubs),
	remove_link(Last, Years, 'BT', BTSubs),
	% relink BT
	sub_root(Last, Did),
	add_link(Did, Years, 'BT', BTSubs),
	% add object trace
	feature(Last, position=LastPos),
	rightendpos(Last, [], TracePos),
	add_token([], [position=TracePos, bar=0], Trace),
	% link object trace
	add_link(Last, Trace, 'OT', []),
	add_link(Trace, Years, 'Trace', []).

% CC
% - conflicts with the projectivity constraint (after change of W)
% - relink
%relink:-
%	link(SubjNr, DepNr, 'CC', CCSubs),
%	link(VerbNr, SubjNr, 'S', SSubs),
%	remove_link(SubjNr, DepNr, 'CC', CCSubs),
%	add_link(VerbNr, DepNr, 'CC', CCSubs).

% CO
% - conflicts with the root word constraint
% - change of direction / relink
%relink:-
%	link(OpenerNr, SubjNr, 'CO', COSubs),
%	link(VerbNr, SubjNr, 'S', _SSubs),
%	remove_link(OpenerNr, SubjNr, 'CO', COSubs),
%	add_link(VerbNr, OpenerNr, 'CO', COSubs).

% EF - change direction
relink:-
	link(H, D, 'EF', S),
	remove_link(H, D, 'EF', S),
	add_link(D, H, 'EF', S).

% NJ
%relink:-
%	link(H, D, 'NJ', S),
%	remove_link(H, D, 'NJ', S),
%	add_link(D, H, 'NJ', S).

% Qd - remove
relink:-
	link(HeadNr, DepNr, 'Q', ['d'|_]),
	remove_link(HeadNr, DepNr, 'Q', ['d'|_]).

% RW - remove
relink:-
	link(HeadNr, DepNr, 'RW', Subs),
	remove_link(HeadNr, DepNr, 'RW', Subs).

% W - relink
%relink:-
%	link(HeadNr, SubjNr, 'W', WSubs),
%	link(VerbNr, SubjNr, 'S', SSubs),
%	remove_link(HeadNr, DepNr, 'W', WSubs),
%	add_link(HeadNr, VerbNr, 'W', WSubs).

% X - remove
relink:-
	link(HeadNr, DepNr, 'X', Subs),
	remove_link(HeadNr, DepNr, 'X', Subs).

relink. % default. do not remove this line.
